#include "taskFlyport.h"



//LED STATUS
#define   STATUS_LED_ON         IOPut(D5Out , 1)   //OUT4 Net su flyport
#define   STATUS_LED_OFF        IOPut(D5Out , 0)
#define   STATUS_LED_TOGGLE     IOPut(D5Out , IOGet(D5Out) ^ 0x01)

//LED 1
#define   LED1_ON               IOPut(p4 , 1)  //IOPut(D5Out , 1)   //OUT5 Net su flyport
#define   LED1_OFF              IOPut(p4 , 0) //IOPut(D5Out , 0)
#define   LED1_TOGGLE           IOPut(p4 , IOGet(p4) ^ 0x01) //IOPut(D5Out , IOGet(D5Out) ^ 0x01)


#define   DE_485                p2
#define   RE_485                p17
#define   TX_485                p5
#define   RX_485                p7
#define   DEBUG_ON


#define   TX_232          p4
#define   RX_232          p6
#define   CTS_232         p11
#define   RTS_232         p9


//#define RS232_MODE

const int port485   =           2;
char debug[255];
extern int *USTAs[];

void InitRS485(){   //Initi porta RS485
  UARTOff(port485);
  #ifdef RS232_MODE
	  UARTWrite(1,"232 Mode\r\n");
	  IOInit(TX_232, UART2TX);
	  IOInit(RX_232, UART2RX);
	  IOInit(CTS_232, out);
	  IOPut (CTS_232, off);
	  IOInit(RTS_232, out);
	  IOPut (RTS_232, off);
  #else
      UARTWrite(1,"485 Mode\r\n");
	  IOInit(TX_485, UART2TX);
	  IOInit(RX_485, UART2RX);
	  IOInit(RX_485, in);
	  IOInit(RE_485, out);
	  IOPut (RE_485, off);
	  IOInit(DE_485, out);
	  IOPut (DE_485, off);
  #endif
  UARTInit(port485, 19200);
  UARTOn(port485);
}


char ModbusRxData[64];

unsigned int ModbusCRC(char *data, int datalen){
  int i,j;
  unsigned int c, crc=0xFFFF;
  for (i=0;i<datalen;i++) {
    c=data[i] & 0x00FF;
    crc^=c;
    for(j=0;j<8;j++) { 
      if (crc & 0x0001){
        crc>>=1;
        crc^=0xA001;
      } else crc>>=1;
    }
  }
  return(crc);
}

unsigned int ModbusSendCmd(int address, int function, int regaddr,char *regvalue,  int registerqt, char bytecount){
  char txcmd[64];
  unsigned int crccmd;
  unsigned int rxsize=0, rxsizetmp=0;
  unsigned int txsize=0;
  unsigned char i;
  unsigned char t;
  UARTFlush(port485);
  DelayMs(100);
  txcmd[0]=address & 0x00FF;   //Confeziono il comando
  txcmd[1]=function & 0x00FF;
  txcmd[2]=((regaddr>>8) & 0x00FF);
  txcmd[3]=(regaddr & 0x00FF);
  if(registerqt>1) {
    txcmd[4] = ((registerqt>>8) & 0x00FF);
    txcmd[5] = (registerqt & 0x00FF);
    if(bytecount) txcmd[6] = bytecount;
    memcpy(txcmd+7,regvalue, bytecount);
    crccmd = ModbusCRC(txcmd,(7+bytecount));
    txcmd[7+bytecount] = (crccmd & 0x00FF);
    txcmd[7+bytecount+1] = ((crccmd>>8)& 0x00FF);
    txsize = 7+bytecount+2;
  } else {
    txcmd[4] = regvalue[0];
    txcmd[5] = regvalue[1];
    crccmd = ModbusCRC(txcmd,6);
    txcmd[6] = (crccmd & 0x00FF);
    txcmd[6+1] = ((crccmd>>8)& 0x00FF);
    txsize = 8;
  }
  #ifdef DEBUG_ON
	  UARTWrite(1,"TX ->");
	  for (i=0; i<txsize; i++) {
		sprintf(debug, " 0x%02x",(unsigned char)txcmd[i]);
		UARTWrite(1,debug);
	  }
	  UARTWrite(1,"\r\n");
  #endif
  
  IOPut(DE_485, on);   //Trasmetto comando
  IOPut(RE_485, on);
  for (i=0; i<txsize; i++) UARTWriteCh(port485, txcmd[i]);
  while ((*USTAs[1] & 256)==0);  //!! Attende la fine della trasmissione
  IOPut(DE_485, off); //Cambio stato delle linee
  IOPut(RE_485, off);
  rxsizetmp=0;   //Attendo Risposta
  while(1){
    DelayMs(10);
    rxsize = UARTBufferSize(port485);
    if(rxsize!=rxsizetmp) {
      rxsizetmp = rxsize;
      t=0;
    } else t++;
    if(t>5){
      if(rxsize>0) {
        UARTRead(port485, ModbusRxData, rxsize);
        crccmd=ModbusCRC(ModbusRxData,rxsize-2);         //Verifica CRC
        if(((unsigned char)(crccmd&0x00ff) == (unsigned char)ModbusRxData[rxsize-2]) && ((unsigned char)((crccmd>>8)&0x00ff)== (unsigned char)ModbusRxData[rxsize-1])){
          #ifdef DEBUG_ON
			  UARTWrite(1,"RX <- ");
			  for (i=0; i<rxsize; i++) {
				sprintf(debug, " 0x%02x",(unsigned char)ModbusRxData[i]);
				UARTWrite(1,debug);
			  }
			  UARTWrite(1,"\r\n");
          #endif
          return (rxsize);
          
          
        } else 
          return (-1); //CRC errato
      } 
      return 0;
    }
  }
}  

void FlyportTask() {
  unsigned int rxsize=0;
  char txcmd[100];
  int i;
  int adc;
  
  UARTWrite(1,"***********************\r\n");
  UARTWrite(1,"Test Flyport EXP485    \r\n");
  UARTWrite(1,"Dev by DeSantiX.iT     \r\n");
  UARTWrite(1,"***********************\r\n");
  
  InitRS485();
  
  while(1){
    UARTWrite(1,"Test controllo singolo rele'\r\n");
    for (i=0; i<8; i++) {
      txcmd[0] = 0xFF;
      txcmd[1] = 0x00;
      rxsize=ModbusSendCmd(0x01, 0x05, i, txcmd,  0x0001, 0x02);
      if(rxsize>4){
        sprintf(debug, "Rele' %d Chiuso \r\n",(i+1) );
        UARTWrite(1,debug);
      }
      DelayMs(500);
      txcmd[0] = 0x00;
      rxsize=ModbusSendCmd(0x01, 0x05, i, txcmd,  0x0001, 0x02);
      if(rxsize>4){
        sprintf(debug, "Rele' %d Aperto \r\n",(i+1) );
        UARTWrite(1,debug);
      }
    }
    
    UARTWrite(1,"Test controllo contemporaneo 8 rele'\r\n");
    txcmd[0] = 0xFF;
    txcmd[1] = 0x00;
    rxsize= ModbusSendCmd(0x01, 0x0F, 0x0000, txcmd, 0x0008, 0x01);
    if(rxsize>4){
      sprintf(debug, "Tutti i rele' chiusi");
      UARTWrite(1,debug);
    }
    DelayMs(500);
    txcmd[0] = 0x00;
    rxsize= ModbusSendCmd(0x01, 0x0F, 0x0000, txcmd, 0x0008, 0x01);
    if(rxsize>4){
      sprintf(debug, "Tutti i rele' aperti");
      UARTWrite(1,debug);
    }
    
    UARTWrite(1,"Lettura stato degli 8 rele'\r\n");
    txcmd[0] = 0x00;
    txcmd[1] = 0x01;
    rxsize= ModbusSendCmd(0x01, 0x01, 0x0000, txcmd, 0x0001, 0x02);
    if(rxsize>4){
      for (i=0; i<7; i++) {
        sprintf(debug, "-rele' %d = %d\r\n" , (i+1), ((ModbusRxData[3]>>i) & 0x01));
        UARTWrite(1,debug);
      }
    }
    UARTWrite(1,"**************************\r\n");
    
    
    
    UARTWrite(1,"Lettura stato ingressi\r\n");
    txcmd[0] = 0x00;
    txcmd[1] = 0x01;
    rxsize= ModbusSendCmd(0x01, 0x04, 0x0000, txcmd, 0x0001, 0x02);
    adc = ModbusRxData[4] + (ModbusRxData[3]<<8);
    for (i=0; i<7; i++) {
      sprintf(debug, "-input' %d = %d\r\n" , (i+1), ((ModbusRxData[4]>>i) & 0x01));
      UARTWrite(1,debug);
    }
    UARTWrite(1,"**************************\r\n");
    
    UARTWrite(1,"Lettura linee ADC\r\n");
    UARTWrite(1,"(ponticelli aperti su EXP485)\r\n");
    txcmd[0] = 0x00;
    txcmd[1] = 0x04;
    rxsize= ModbusSendCmd(0x01, 0x04, 0x0001, txcmd, 0x0001, 0x02);
    if(rxsize>4){
      adc = ModbusRxData[4] + (ModbusRxData[3]<<8);
      sprintf(debug, "ADC1 %.1f V [%d]\r\n" , (double)(adc * 0.01), adc);
      UARTWrite(1,debug);
      adc = ModbusRxData[6] + (ModbusRxData[5]<<8);
      sprintf(debug, "ADC2 %.1f V [%d]\r\n" , (double)(adc * 0.01), adc);
      UARTWrite(1,debug);
      adc = ModbusRxData[8] + (ModbusRxData[7]<<8);
      sprintf(debug, "ADC3 %.1f V [%d]\r\n" , (double)(adc * 0.01), adc);
      UARTWrite(1,debug);
      adc = ModbusRxData[10] + (ModbusRxData[9]<<8);
      sprintf(debug, "ADC4 %.1f V [%d]\r\n" , (double)(adc * 0.01), adc);
      UARTWrite(1,debug);
  }
    
    
    vTaskDelay(1000);
  }


}
